/* 
 * VoiceControl Plugin version 0.1
 * For Amarok 2 Music Player
 * Authors: Alberto Testolin
 *          Alberto Zuccato
 * Last modified:   29-11-2010
 * License: GPL v2
 *
 * Main file to control the plugin.
 */

Importer.loadQtBinding( "qt.core" );
Importer.loadQtBinding( "qt.gui" );

// global variable to synchronize the main program and the configuration manager
ready = false;

Importer.include("firstRun.js");

// process to start and control Sphinx
myProcess = new QProcess(this);

function start(){
  // wait until the configuration path has been set
  if (ready){
    path = Amarok.Info.scriptPath();
    Amarok.debug("------- ready: " + ready);
    myProcess.start("sh " + path + "/launcher.sh");
    if(myProcess.waitForStarted(5000) && (myProcess.exitStatus() == 0)){
      this.timer = new QTimer(this);
      var y = this;
      this.timer.timeout.connect(function() { y.listen();});
      this.timer.start(300);
      this.wait_configuration_timer.stop();
  }
  else
    Amarok.alert("Failed to start Sphinx. Check out your configuration.");
  }
}

// recognize vocal commands
function listen(){
  res = myProcess.readAllStandardOutput();
	ts = new QTextStream( res, QIODevice.ReadOnly );
	cmd = ts.readAll();
	if (cmd != ""){
	  if (cmd == "amarok play"){
	    
	    // command "play" can work also for changing the Mute state
	    if (Amarok.Engine.volume == 0)
	      Amarok.Engine.Mute();
	    
	    Amarok.Engine.Play();
	  }
	    
	  if (cmd == "amarok stop")
	    Amarok.Engine.Stop();
	    
	  if (cmd == "amarok pause")
	    Amarok.Engine.Pause();
	    
	  if (cmd == "amarok skip track")
	    Amarok.Engine.Next();
	    
	  if (cmd == "amarok previous track")
	    Amarok.Engine.Prev();
	    
	  if (cmd == "amarok raise volume")
	    Amarok.Engine.IncreaseVolume(15);
	    
	  if (cmd == "amarok decrease volume")
	    Amarok.Engine.DecreaseVolume(15);
	    
	  if (cmd == "amarok shut up")
	    Amarok.Engine.Mute();
	    
	  if (cmd == "amarok rate song one")
	    Amarok.Engine.currentTrack().rating = 2;
	    
	  if (cmd == "amarok rate song two")
	    Amarok.Engine.currentTrack().rating = 4;
	    
	  if (cmd == "amarok rate song three")
	    Amarok.Engine.currentTrack().rating = 6;
	    
	  if (cmd == "amarok rate song four")
	    Amarok.Engine.currentTrack().rating = 8;
	    
	  if (cmd == "amarok rate song five")
	    Amarok.Engine.currentTrack().rating = 10;
	    
	  if (cmd == "amarok create new playlist")
  	  Amarok.Playlist.clearPlaylist();
  	  
  	// FIXXX: read absolute path of tracks instead of relative
	  if (cmd == "amarok favorite songs"){
	    
	    var urls = Amarok.Collection.query( "SELECT u.rpath FROM statistics AS s " +
	                                        "INNER JOIN urls AS u ON s.url = u.id " +
	                                        "WHERE s.rating >= 8 LIMIT 50;" );
      for ( var i in urls ){
        var track_path = "file://";
        var str = urls[ i ];
        str_normalized = str.substr(1);
        track_path = track_path + str_normalized;
        Amarok.Playlist.addMedia( new QUrl( track_path ) );
        
      }
      Amarok.debug(track_path);
      
	  }
	  
	  if (cmd == "amarok recent songs"){
	    var urls = Amarok.Collection.query( "SELECT u.rpath FROM statistics AS s " +
	                                        "INNER JOIN urls AS u ON s.url = u.id " +
	                                        "ORDER BY s.accessDate DESC LIMIT 50;" );
      for ( var i in urls ){
        var path = "file://";
        var str = urls[ i ];
        str_normalized = str.substr(1);
        path = path + str_normalized;
        Amarok.Playlist.addMedia( new QUrl( path ) );
      }
	  }
	  
	  if (cmd == "amarok random songs"){
	    var n = Amarok.Collection.query( "SELECT COUNT(t.id) FROM tracks AS t;" );
      var tot = 50;
      var current = new Array(tot);
      var count = 0;
      var found = false;
      // generate tot unique random numbers between tracks ids
      for (var i = 0; count < tot; count++){
        found = false;
        var randomnumber = Math.floor(Math.random()*n) + 1;
        for (var j = 0; j < current.length; j++){
          if (current[j] == randomnumber){
            found = true;
            break
          }
        }
        if (found)
          count--
        else
          current[count] = randomnumber;
      }
      for ( var i in current ){
        var url = Amarok.Collection.query( "SELECT u.rpath FROM urls AS u WHERE u.id = " + current[i] + ";" );
        var path = "file://";
        var str = url[0];
        str_normalized = str.substr(1);
        path = path + str_normalized;
        Amarok.Playlist.addMedia( new QUrl( path ) );
      }
	  }
	  
	  if (cmd == "amarok add artist amon tobin"){
      var urls = Amarok.Collection.query( "SELECT u.rpath FROM tracks AS t " +
                                          "INNER JOIN urls AS u ON t.url = u.id " +
                                          "WHERE t.artist IN " +
                                          "(SELECT a.id FROM artists AS a WHERE a.name = 'Amon Tobin');" );
      for ( var i in urls ){
        var path = "file://";
        var str = urls[ i ];
        str_normalized = str.substr(1);
        path = path + str_normalized;
        Amarok.Playlist.addMedia( new QUrl( path ) );
      }
	  }
	  
	  if (cmd == "amarok add genre ambient"){
	    var urls = Amarok.Collection.query( "SELECT u.rpath FROM tracks AS t " +
	                                        "INNER JOIN urls AS u ON t.url = u.id " +
	                                        "WHERE t.artist IN " +
	                                        "(SELECT g.id FROM genres AS g WHERE g.name = 'Ambient');" );
      for ( var i in urls ){
        var path = "file://";
        var str = urls[ i ];
        str_normalized = str.substr(1);
        path = path + str_normalized;
        Amarok.Playlist.addMedia( new QUrl( path ) );
      }
	  }
	  
	  if (cmd == "amarok add artist bob marley"){
      var urls = Amarok.Collection.query( "SELECT u.rpath FROM tracks AS t " +
                                          "INNER JOIN urls AS u ON t.url = u.id " +
                                          "WHERE t.artist IN " +
                                          "(SELECT a.id FROM artists AS a WHERE a.name = 'Bob Marley');" );
      for ( var i in urls ){
        var path = "file://";
        var str = urls[ i ];
        str_normalized = str.substr(1);
        path = path + str_normalized;
        Amarok.Playlist.addMedia( new QUrl( path ) );
      }
	  }
	  
	  if (cmd == "amarok add genre reggae"){
	    var urls = Amarok.Collection.query( "SELECT u.rpath FROM tracks AS t " +
	                                        "INNER JOIN urls AS u ON t.url = u.id " +
	                                        "WHERE t.artist IN " +
	                                        "(SELECT g.id FROM genres AS g WHERE g.name = 'Reggae');" );
      for ( var i in urls ){
        var path = "file://";
        var str = urls[ i ];
        str_normalized = str.substr(1);
        path = path + str_normalized;
        Amarok.Playlist.addMedia( new QUrl( path ) );
      }
	  }
	  
	  Amarok.debug(cmd);
	}
}


// stop the recognition process when exiting Amarok (teo giuro)
function onExit(){
    Amarok.debug("killing processes...");
    //myProcess.kill();
    pid = QCoreApplication.applicationPid();
    pid += '';
    killer = new QProcess();
    params = new QStringList();
    params.append("--ppid");
    params.append(pid);
    params << "-o";
    params << "pid";
    params << "--noheaders";
    killer.start("/bin/ps", params);
    // killer doesn't start! (problems with QStringList)
    Amarok.debug("killer started");
    if(killer.waitForStarted(-1)){
      if(killer.waitForFinished(-1)){
        temp = killer.readAllStandardOutput();
        //str = QString::fromLocal8Bit(temp);
        ts = new QTextStream( temp, QIODevice.ReadOnly );
	      str = ts.readAll();
        list = str.split("\n");
        for(i = 0; i < list.size(); i++){
          if(!list.at(i).isEmpty())
            kill(list.at(i).toInt(), SIGKILL);
        }
      }
    }
}


// before starting, wait for the configuration to be set
this.wait_configuration_timer = new QTimer(this);
var x = this;
this.wait_configuration_timer.timeout.connect(function() { x.start();});
this.wait_configuration_timer.start(1000);


// perform clean-up when Amarok quit
QCoreApplication.instance().aboutToQuit.connect(onExit);


